/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/* SCCS ID: @(#)dev_match.c	1.7 1/13/94 */
/*************************************************************************
 * FILE:      dev_match.c
 * Author:    Claude Saunders
 * Purpose:   Implements the devMatch device access library call. The
 *            device access library invokes the device_match_1() remote
 *            procedure call which is serviced here.
 *
 * Mods:      login mm/dd/yy: description
 *************************************************************************/
#include <stdio.h>
#include <rpc/rpc.h>
#include <string.h>
#include <stdlib.h>
#include "dev.h"
#include "scan_defs.h"

int patmatch(char *pattern, char *string);

/*************************************************************************
 * FUNCTION : device_match_1()
 * PURPOSE  : Given a device name (which may contain the wildcard characters
 *            '*' and '?'), return a linked list of all device names which
 *            match the pattern.
 *
 * ARGS in  : pattern: string containing device name with '*' and '?' chars
 * ARGS out : none
 * GLOBAL   : scans server hash table
 * RETURNS  : singly linked list of device names (see RPC interface file dev.x)
 ************************************************************************/
namelist *device_match_1(pattern)
     char **pattern;
{
  PDEV pdev;                  /* device header node */
  PPVMSG ppvmsg;              /* ptr to linked list of supported messages */
  PMSG pmsg;                  /* ptr to message specific information node */
  static namelist res;        /* RPC return MUST be static ! */
  namelist list = NULL;       /* list to be returned */
  namelist prevlist = NULL;
  namelist listhead = NULL;

  xdr_free(xdr_int,(char *)&res);  /* free up return from last call */

  /* Dev_next() is used to pass through the whole hash table, 
     retrieving one entry at a time. Dev_next(0) essentially
     rewinds the search. Dev_next(1) gets the next entry.
  */
  dev_next(0);
  
  /* For each device name in hash table (atomic and composite names) */
  while ((pdev = dev_next(1)) != NULL) {
    if (patmatch(*pattern,pdev->name)) {  /* if device name matches pattern */
      list = (namelist)malloc(sizeof(struct namenode));
      if (prevlist == NULL)
	listhead = list;
      else
	prevlist->next = list;
      list->next = NULL;
      prevlist = list;
      list->dname = strdup(pdev->name);
    }
  }
  res = listhead;
  return(&res);
}

