/* SCCS ID: @(#)dev.y	1.10 10/12/93 */
/*
 * dev_yacc.y,v
 * Revision 1.6  2005/03/01 22:26:27  soliday
 * Updated to work with Epics/Base 3.14.7
 *
 * Revision 1.5  2004/02/26 16:53:41  jba
 * Changes to avoid identifier redeclared: for MSG and CLASS.
 *
 * Revision 1.4  2000/10/24 20:43:30  jba
 * Fixed comments and made some ansi c changes.
 *
 * Revision 1.3  1996/10/02 18:51:31  saunders
 * Restored use of lex and yacc, since flex and antelope aren't
 * handling the grammar for some reason. Also touched up some
 * compiler warnings.
 *
 * Revision 1.2  1994/12/13 19:46:54  jba
 * Conversion to antelope and e_flex
 *
 * Revision 1.1  1994/09/21  22:19:04  jba
 * New definitions in CONFIG* files
 *
 * Revision 1.2  1994/05/11  20:17:49  saunders
 * First fully functional version added to cvs addon environment.
 * */
/*************************************************************************/
/* FILE:      dev.y                                                      */
/* Author:    Claude Saunders                                            */
/* Purpose:   yacc input file                                            */
/*            Defines device definition grammar. This includes the       */
/*            expressions for class definition (.cl files), atomic       */
/*            instance definition (.at files), and composite instance    */
/*            definition (.co files). Data structures representing this  */
/*            information are built up on the fly as expressions are     */
/*            recognized by the parser. Assorted semantic errors         */
/*            are checked for in addition to usual sytax error checking. */
/*                                                                       */
/* Mods:      login mm/dd/yy: description                                */
/*************************************************************************/
/* SCCS ID: @(#)dev.y	1.10 10/12/93 */
%{
#include <stdio.h>
#include <memory.h>
#include <string.h>
static int yy_start;
int myParse();
#include <db_access.h>
#include "builderyacc.h"
#include "scan_defs.h"

int logerror(char *msg1,char *msg2);

#define BOOL char
#define FALSE 0
#define TRUE 1

int scanned_atomic = 0;       /* Set to true when done scanning .cl and .at */
char scan_error_msg[255];     /* composite scan error msgs stashed here     */
static char buf[255];
static int line_num;

static PCLASS pclass;         /* class definition structure */
static PDEV pdev;             /* device header node                       */
static PPVMSG ppvmsg;         /* ptr to linked list of supported messages */
static PMSG pmsg;             /* message specific information node        */

%}

%start devices

%token <Str> COMMA
%token <Str> WORD VALUE 
%token <Str> FIELD
%left O_BRACE C_BRACE O_PAREN C_PAREN COLON
%left MYCLASS INST MYMSG COMP
%left DIR TYPE ARGS DEFAULT PV
%left WORD
%left NOWHERE 

%union
{
    int	Int;
    char Char;
    char *Str;
    double Real;
}

%%

devices:
  /* null */
    {
      strcpy(scan_error_msg,"file is empty");
      logerror("parse",scan_error_msg);
    }
  |   { scan_error_msg[0] = '\0'; } classes instances
  ;

 classes:
  /* null */
  |
  classes class
  ;

 instances:
  /* null */
  |
  instances instance
  ;

 class:         /* device class definition expression */
  MYCLASS WORD 
    {
      if (scanned_atomic) {
	strcpy(scan_error_msg,"cannot redefine class once server started");
	logerror("class",scan_error_msg);
	YYERROR;
      }
      pclass = allocateClass();
      strcpy(pclass->name,$<Str>2);
      free($<Str>2);
    }
  O_BRACE class_msgs C_BRACE
    {
    }
  ;

 instance:
  INST WORD COLON COMP                   /* composite instance expression */
    {
      if (dev_find($<Str>2) != NULL)     /* allow for redefinition */
	dev_delete($<Str>2);
      pdev = dev_add($<Str>2);
      strcpy(pdev->name,$<Str>2);
      pdev->type = COMPOSITE;
      strcpy(pdev->class,"composite");
      pdev->pvmsgh = NULL;
    }
  O_BRACE device_list C_BRACE
    {
      free($<Str>2);
    }
  | INST WORD COLON WORD                 /* atomic instance expression */
    {
      if (scanned_atomic) {
	strcpy(scan_error_msg,"cannot redefine atomics once server started");
	logerror("inst",scan_error_msg);
	YYERROR;
      }
      pclass = findClass($<Str>4);
      if (pclass == NULL) {
	sprintf(buf,"line %d class %s not defined",line_num,$<Str>4);
	logerror("inst",buf);
	YYERROR;
      }
      pdev = dev_add($<Str>2);
      strcpy(pdev->name,$<Str>2);
      strcpy(pdev->class,pclass->name);
      pdev->type = ATOMIC;
    }
  O_BRACE inst_msgs C_BRACE
    {
      free($<Str>2);
      free($<Str>4);
    }
  ;
  
class_msgs:
  /* null */
  | class_msgs class_msg
  ;

inst_msgs:
  /* null */
  | inst_msgs inst_msg
  ;

device_list:
  /* null */
  | device_list device
  ;

class_msg:        /* definition of new message supported by a device class */
  MYMSG WORD O_BRACE DIR WORD ARGS WORD DEFAULT WORD TYPE WORD C_BRACE
    {
      pmsg = allocateMsg(pclass);
      strcpy(pmsg->name,$<Str>2);
      pmsg->dir = (strcmp($<Str>5,"r"))?WRITE:READ;
      pmsg->nargs = atoi($<Str>7);
      if (pmsg->nargs != 0) {
	sprintf(buf,"line %d args must be 0 when default value used",line_num);
	logerror("class",buf);
	YYERROR;
      }
      if (pmsg->dir != WRITE) {
	sprintf(buf,"line %d dir can be w only when args is 0",line_num);
	logerror("class",buf);
	YYERROR;
      }
      strcpy(pmsg->def,$<Str>9);
      if (!strcmp($<Str>11,"DBR_DOUBLE"))
	pmsg->type = DBR_DOUBLE;
      else if (!strcmp($<Str>11,"DBR_STRING"))
	pmsg->type = DBR_STRING;
      else {
	sprintf(buf,"line %d %s is not a valid data type",line_num,$<Str>11);
	logerror("class",buf);
	YYERROR;
      }
      free($<Str>2);
      free($<Str>5);
      free($<Str>7);
      free($<Str>9);
      free($<Str>11);
    }
  | MYMSG WORD O_BRACE DIR WORD ARGS WORD TYPE WORD C_BRACE
    {
      pmsg = allocateMsg(pclass);
      strcpy(pmsg->name,$<Str>2);
      if (strcmp($<Str>5,"r") && strcmp($<Str>5,"w")) {
	sprintf(buf,"line %d dir must be r or w only",line_num);
	logerror("class",buf);
	YYERROR;
      }
      pmsg->dir = (strcmp($<Str>5,"r"))?WRITE:READ;
      pmsg->nargs = atoi($<Str>7);
      if (pmsg->nargs != 1) {
	sprintf(buf,"line %d args must be 1 when no default value used",line_num);
	logerror("class",buf);
	YYERROR;
      }      
      if (!strcmp($<Str>9,"DBR_DOUBLE"))
	pmsg->type = DBR_DOUBLE;
      else if (!strcmp($<Str>9,"DBR_STRING"))
	pmsg->type = DBR_STRING;
      else {
	sprintf(buf,"line %d %s is not a valid data type",line_num,$<Str>9);
	logerror("class",buf);
	YYERROR;
      }
      strcpy(pmsg->def,"0.0");
      free($<Str>2);
      free($<Str>5);
      free($<Str>7);
      free($<Str>9);
    }
  ;

inst_msg:             /* specification of pv for a given message */
  MYMSG WORD O_BRACE PV WORD C_BRACE
    {
      ppvmsg = allocatePVMsg(pdev);
      strcpy(ppvmsg->pvname,$<Str>5);

      pmsg = findMsg(pclass,$<Str>2);
      if (pmsg == NULL) {
	sprintf(buf,"line %d no %s msg defined for %s class",line_num,$<Str>2,pclass->name);
	logerror("inst",buf);
	YYERROR;
      }
      ppvmsg->msgh = pmsg;

      free($<Str>2);
      free($<Str>5);
    }
  ;

device:            /* specification of a constituent device of a composite */
  WORD
    {
      PDEV atomicdev;
      pdev = allocateDev(pdev);
      atomicdev = dev_find($<Str>1);
      if (atomicdev == NULL || atomicdev->type != ATOMIC) {
	sprintf(buf,"line %d atomic device %s not defined",line_num,$<Str>1);
	logerror("comp",buf);
	strcpy(scan_error_msg,buf);
	YYERROR;
      }
      strcpy(pdev->name,atomicdev->name);
      pdev->type = atomicdev->type;
      strcpy(pdev->class,atomicdev->class);
      pdev->pvmsgh = atomicdev->pvmsgh;
      free($<Str>1);
    }
  ;
%%
 
#include "dev_lex.c"
 
int yyerror(str)
char  *str;
{
  char buf[255];
  sprintf(buf,"line %d text %s",line_num,yytext);
  strcpy(scan_error_msg,"syntax error: ");
  strcat(scan_error_msg,buf);
  logerror("syntax",buf);
  return(0);
}

int myParse()
{
	yyreset();
	return(yyparse());
}

int yywrap() {
   return(1);
}

