/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/**** SCCS ID: @(#)logentry.c	1.4 1/14/94 ****/
/*********************************************************************
 * FILE:      logentry.c
 * FUNCTIONS: logerror(),logclient(),logserver()
 * Purpose:   General utility functions for logging info to stderr.
 *            Provides header information (eg. host, date, etc)
 *********************************************************************/
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <time.h>
#include <sys/time.h>
#include <syslog.h>
#include <strings.h>
#include <unistd.h>

#define USESYSLOG 1

/*************************************************************************
 * FUNCTION : logerror()
 * PURPOSE  : Appends ERROR entry to stderr.
 * ARGS in  : none
 * ARGS out : none
 * GLOBAL   : writes to stderr
 * RETURNS  : nothing
 ************************************************************************/
int logerror(msg1,msg2)
     char *msg1, *msg2;
{
  int pid;
  char hostname[256];
  time_t currtime;
  char *timeptr;
  char timestr[256];
  char msgbuf[256];
  int lockd;


  if (USESYSLOG) {
    sprintf(msgbuf,"ERROR|%s|%s|",msg1,msg2);
    syslog(LOG_ERR,msgbuf);
  } else {
    gethostname(hostname,255);
    pid = getpid();
    time(&currtime);
    timeptr = ctime(&currtime);
    strncpy(timestr,timeptr,strlen(timeptr)-1);
    timestr[strlen(timeptr)-1] = '\0';
    fprintf(stderr,
	    "|dev_svc|ERROR |%s|%d|%s|%s|%s\n",hostname,pid,timestr,msg1,msg2);
    fflush(stderr);
  }
  return(0);
}

/*************************************************************************
 * FUNCTION : logclient()
 * PURPOSE  : Appends CLIENT entry to stderr.
 * ARGS in  : none
 * ARGS out : none
 * GLOBAL   : writes to stderr
 * RETURNS  : nothing
 ************************************************************************/
int logclient(msg1,msg2)
     char *msg1, *msg2;
{
  int pid;
  char hostname[256];
  time_t currtime;
  char *timeptr;
  char timestr[256];
  int lockd;

    gethostname(hostname,255);
    pid = getpid();
    time(&currtime);
    timeptr = ctime(&currtime);
    strncpy(timestr,timeptr,strlen(timeptr)-1);
    timestr[strlen(timeptr)-1] = '\0';
    fprintf(stderr,
	    "|dev_svc|CLIENT|%s|%d|%s|%s|%s\n",hostname,pid,timestr,msg1,msg2);
    fflush(stderr);
    return(0);
}

/*************************************************************************
 * FUNCTION : logserver()
 * PURPOSE  : Appends SERVER entry to stderr.
 * ARGS in  : none
 * ARGS out : none
 * GLOBAL   : writes to stderr
 * RETURNS  : nothing
 ************************************************************************/
int logserver(msg1,msg2)
     char *msg1, *msg2;
{
  int pid;
  char hostname[256];
  time_t currtime;
  char *timeptr;
  char timestr[256];
  char msgbuf[255];
  int lockd;

  if (USESYSLOG) {
    sprintf(msgbuf,"INFO|%s|%s|",msg1,msg2);
    syslog(LOG_INFO,msgbuf);
  } else {
    gethostname(hostname,255);
    pid = getpid();
    time(&currtime);
    timeptr = ctime(&currtime);
    strncpy(timestr,timeptr,strlen(timeptr)-1);
    timestr[strlen(timeptr)-1] = '\0';
    fprintf(stderr,
	    "|dev_svc|INFO |%s|%d|%s|%s|%s\n",hostname,pid,timestr,msg1,msg2);
    fflush(stderr);
  }
  return(0);
}
